==> audiochain-1.1.5-src.tgz: org/audiochain/ui/gui/mixer/KeyEventManager.java <==
/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.ui.gui.mixer;

import static java.awt.event.KeyEvent.*;

import java.awt.Component;
import java.awt.KeyEventDispatcher;
import java.awt.event.KeyEvent;
import java.io.IOException;

import javax.swing.FocusManager;
import javax.swing.JDialog;
import javax.swing.JTextField;

import org.audiochain.io.AudioIOException;
import org.audiochain.model.AudioProject;
import org.audiochain.model.AudioTrack;
import org.audiochain.model.PerformanceManager;
import org.audiochain.ui.gui.Dialog;

public class KeyEventManager implements KeyEventDispatcher {

	private final GuiAudioRecorder guiAudioRecorder;
	private StringBuilder typedKeys;

	public KeyEventManager(GuiAudioRecorder guiAudioRecorder) {
		this.guiAudioRecorder = guiAudioRecorder;
	}

	@Override
	public boolean dispatchKeyEvent(KeyEvent evt) {
		try {
			return handleEvent(evt);
		} catch (IOException ex) {
			ex.printStackTrace();
			Dialog.message(guiAudioRecorder, "IO Error", ex);
			return false;
		} catch (InstantiationException ex) {
			throw new IllegalStateException(ex);
		} catch (IllegalAccessException ex) {
			throw new IllegalStateException(ex);
		} catch (AudioIOException ex) {
			ex.printStackTrace();
			Dialog.message(guiAudioRecorder, "Audio Mixer Error", ex);
			return false;
		}
	}

	private boolean handleEvent(KeyEvent e) throws IOException, InstantiationException, IllegalAccessException, AudioIOException {
		if (e.getID() == KEY_PRESSED) {
			Component focusOwner = FocusManager.getCurrentManager().getFocusOwner();
			if (!hasSameParentFrame(focusOwner)) {
				return false;
			}
			if (focusOwner instanceof JTextField) {
				return false;
			}
			AudioProject audioProject = guiAudioRecorder.getAudioProjectLifecycleManager().getActiveAudioProject();
			int modifiersEx = e.getModifiersEx(); // only for keys that are not
													// combined with ctrl, shift
													// or alt
			if (audioProject != null && modifiersEx == 0) {
				int keyCode = e.getKeyCode();
				PerformanceManager performanceManager = audioProject.getPerformanceManager();
				switch (keyCode) {
				case VK_ESCAPE:
					if (performanceManager.isRunning()) {
						performanceManager.stop();
						return true;
					}
					return false;
				case VK_SPACE:
					performanceManager.toggleStart();
					return true;
				case VK_UP:
					audioProject.setFramePosition(0);
					return false;
				case VK_DOWN:
					long maxFrame = audioProject.getMaxFrame();
					audioProject.setFramePosition(maxFrame);
					return false;
				case VK_LEFT:
					audioProject.decrementFramePosition();
					return false;
				case VK_RIGHT:
					audioProject.incrementFramePosition();
					return false;
				case VK_B:
					performanceManager.startFromBeginning();
					return true;
				case VK_P:
					if (performanceManager.isRunning()) {
						return false;
					}
					performanceManager.start();
					return true;
				case VK_S:
					performanceManager.stop();
					return true;
				case VK_N:
					audioProject.addNewAudioTrack();
					return true;
				}

				// tries to accumulate the typed keys and parse them to a
				// number after a while
				// then select or unselect the track with this number for
				// recording
				char keyChar = e.getKeyChar();
				switch (keyChar) {
				case '1':
				case '2':
				case '3':
				case '4':
				case '5':
				case '6':
				case '7':
				case '8':
				case '9':
				case '0':
					toggleTrackSelection(keyChar);
					break;
				default:
					break;
				}
			}
		}
		return false;
	}

	private boolean hasSameParentFrame(Component component) {
		if (component instanceof JDialog) {
			return false;
		}
		if (component == guiAudioRecorder) {
			return true;
		} else if (component == null) {
			return false;
		}
		return hasSameParentFrame(component.getParent());
	}

	private void toggleTrackSelection(char keyChar) {
		if (typedKeys == null) {
			typedKeys = new StringBuilder();
			new Thread() {
				@Override
				public void run() {
					try {
						sleep(750);
						int number = Integer.parseInt(typedKeys.toString());
						AudioProject audioProject = guiAudioRecorder.getAudioProjectLifecycleManager().getActiveAudioProject();
						for (AudioTrack track : audioProject.getAudioTracks()) {
							if (track.getNumber() == number) {
								track.toggleSelectForRecording();
							}
						}
					} catch (NumberFormatException ex) {
						// nop
					} catch (InterruptedException e) {
						// nop
					} catch (IOException e) {
						Dialog.message(this, "Track Selection: IO Error", e);
					} catch (AudioIOException e) {
						Dialog.message(this, "Track Selection: Audio Mixer Error", e);
					} finally {
						typedKeys = null;
					}
				}
			}.start();
		}
		typedKeys.append(keyChar);
	}
}

==> audiochain-1.1.6-src.tgz: org/audiochain/ui/gui/mixer/KeyEventManager.java <==
/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.ui.gui.mixer;

import static java.awt.event.KeyEvent.KEY_PRESSED;
import static java.awt.event.KeyEvent.VK_B;
import static java.awt.event.KeyEvent.VK_DOWN;
import static java.awt.event.KeyEvent.VK_ESCAPE;
import static java.awt.event.KeyEvent.VK_LEFT;
import static java.awt.event.KeyEvent.VK_N;
import static java.awt.event.KeyEvent.VK_P;
import static java.awt.event.KeyEvent.VK_RIGHT;
import static java.awt.event.KeyEvent.VK_S;
import static java.awt.event.KeyEvent.VK_SPACE;
import static java.awt.event.KeyEvent.VK_UP;

import java.awt.Component;
import java.awt.KeyEventDispatcher;
import java.awt.event.KeyEvent;
import java.io.IOException;

import javax.swing.FocusManager;
import javax.swing.JDialog;
import javax.swing.JTextField;

import org.audiochain.io.AudioIOException;
import org.audiochain.model.AudioProject;
import org.audiochain.model.AudioTrack;
import org.audiochain.model.PerformanceManager;
import org.audiochain.ui.gui.Dialog;

public class KeyEventManager implements KeyEventDispatcher {

	private final GuiAudioRecorder guiAudioRecorder;
	private StringBuilder typedKeys;

	public KeyEventManager(GuiAudioRecorder guiAudioRecorder) {
		this.guiAudioRecorder = guiAudioRecorder;
	}

	@Override
	public boolean dispatchKeyEvent(KeyEvent evt) {
		try {
			return handleEvent(evt);
		} catch (IOException ex) {
			ex.printStackTrace();
			Dialog.message(guiAudioRecorder, "IO Error", ex);
			return false;
		} catch (InstantiationException ex) {
			throw new IllegalStateException(ex);
		} catch (IllegalAccessException ex) {
			throw new IllegalStateException(ex);
		} catch (AudioIOException ex) {
			ex.printStackTrace();
			Dialog.message(guiAudioRecorder, "Audio Mixer Error", ex);
			return false;
		}
	}

	private boolean handleEvent(KeyEvent e) throws IOException, InstantiationException, IllegalAccessException, AudioIOException {
		if (e.getID() == KEY_PRESSED) {
			Component focusOwner = FocusManager.getCurrentManager().getFocusOwner();
			if (!hasSameParentFrame(focusOwner)) {
				return false;
			}
			if (focusOwner instanceof JTextField) {
				return false;
			}
			AudioProject audioProject = guiAudioRecorder.getAudioProjectLifecycleManager().getActiveAudioProject();
			int modifiersEx = e.getModifiersEx(); // only for keys that are not
													// combined with ctrl, shift
													// or alt
			if (audioProject != null && modifiersEx == 0) {
				int keyCode = e.getKeyCode();
				PerformanceManager performanceManager = audioProject.getPerformanceManager();
				switch (keyCode) {
				case VK_ESCAPE:
					if (performanceManager.isRunning()) {
						performanceManager.stop();
						return true;
					}
					return false;
				case VK_SPACE:
					performanceManager.toggleStart();
					return true;
				case VK_UP:
					audioProject.setFramePosition(0);
					return false;
				case VK_DOWN:
					long maxFrame = audioProject.getMaxFrame();
					audioProject.setFramePosition(maxFrame);
					return false;
				case VK_LEFT:
					audioProject.decrementFramePosition();
					return false;
				case VK_RIGHT:
					audioProject.incrementFramePosition();
					return false;
				case VK_B:
					performanceManager.startFromBeginning();
					return true;
				case VK_P:
					if (performanceManager.isRunning()) {
						return false;
					}
					performanceManager.start();
					return true;
				case VK_S:
					performanceManager.stop();
					return true;
				case VK_N:
					audioProject.addNewAudioTrack();
					return true;
				}

				// tries to accumulate the typed keys and parse them to a
				// number after a while
				// then select or unselect the track with this number for
				// recording
				char keyChar = e.getKeyChar();
				switch (keyChar) {
				case '1':
				case '2':
				case '3':
				case '4':
				case '5':
				case '6':
				case '7':
				case '8':
				case '9':
				case '0':
					toggleTrackSelection(keyChar);
					break;
				default:
					break;
				}
			}
		}
		return false;
	}

	private boolean hasSameParentFrame(Component component) {
		if (component instanceof JDialog) {
			return false;
		}
		if (component == guiAudioRecorder) {
			return true;
		} else if (component == null) {
			return false;
		}
		return hasSameParentFrame(component.getParent());
	}

	private void toggleTrackSelection(char keyChar) {
		if (typedKeys == null) {
			typedKeys = new StringBuilder();
			new Thread() {
				@Override
				public void run() {
					try {
						sleep(750);
						int number = Integer.parseInt(typedKeys.toString());
						AudioProject audioProject = guiAudioRecorder.getAudioProjectLifecycleManager().getActiveAudioProject();
						for (AudioTrack track : audioProject.getAudioTracks()) {
							if (track.getNumber() == number) {
								track.toggleSelectForRecording();
							}
						}
					} catch (NumberFormatException ex) {
						// nop
					} catch (InterruptedException e) {
						// nop
					} catch (IOException e) {
						Dialog.message(this, "Track Selection: IO Error", e);
					} catch (AudioIOException e) {
						Dialog.message(this, "Track Selection: Audio Mixer Error", e);
					} finally {
						typedKeys = null;
					}
				}
			}.start();
		}
		typedKeys.append(keyChar);
	}
}
